import 'dart:convert';

import 'package:smart_parking_solutions_common/smart_parking_solutions_common.dart';
import 'package:smart_parking_solutions_common/src/core_objects/parking_space.dart';

///Common object used by front and back end service
///which represents a booking for a parking space
class Booking {
  late int bookingID;
  late User owner;
  late DateTime createdDate;
  late DateTime startDate;
  late DateTime endDate;
  late ParkingSpace bookedSpace;

  Booking(
      {this.bookingID = -1,
      required this.owner,
      required this.createdDate,
      required this.startDate,
      required this.endDate,
      required this.bookedSpace});

  Booking.fromDBObj({required var dbBinary}) {
    bookingID = dbBinary.fields['bookingID'];
    owner = User.fromJson(json: jsonDecode(dbBinary.fields['owner']));
    createdDate = dbBinary.fields['createdDate'];
    startDate = dbBinary.fields['startDate'];
    endDate = dbBinary.fields['endDate'];
    bookedSpace =
        ParkingSpace.fromJson(json: jsonDecode(dbBinary.fields['bookedSpace']));
  }

  Booking.fromJson({required Map<String, dynamic> json}) {
    bookingID = json['bookingID'];
    owner = User.fromJson(json: json['owner']);
    createdDate = DateTime.parse(json['createdDate']);
    startDate = DateTime.parse(json['startDate']);
    endDate = DateTime.parse(json['endDate']);
    bookedSpace = ParkingSpace.fromJson(json: json['bookedSpace']);
  }

  Map toJson() {
    final map = {
      'bookingID': bookingID,
      'owner': owner.toJson(),
      'createdDate': createdDate.toString(),
      'startDate': startDate.toString(),
      'endDate': endDate.toString(),
      'bookedSpace': bookedSpace.toJson()
    };
    return map;
  }

  bool isOverlapping({required Booking booking}) {
    //start is after other startDate and end is before other end date (within)
    if ((booking.startDate.compareTo(startDate) >= 0 &&
            booking.endDate.compareTo(endDate) <= 0) ||
        //start is before other startDate and end is before other endDate and end is after other startDate (overlap from the left)
        (booking.startDate.compareTo(startDate) <= 0 &&
            booking.endDate.compareTo(endDate) <= 0 &&
            booking.endDate.compareTo(startDate) > 0) ||
        //start is after other startDate but before endDate (overlap from the right)
        (booking.startDate.compareTo(startDate) >= 0 &&
            booking.startDate.compareTo(endDate) <= 0) ||
        //start is before startDate and end is before endDate (full overlap)
        (booking.startDate.compareTo(startDate) <= 0 &&
            booking.endDate.compareTo(endDate) >= 0)) {
      return true;
    } else {
      return false;
    }
  }
}
